
----------
March 2016 V 0.8  Doug Johnson (finson@whidbey.com) 

----------

## DeviceFeature and Device Drivers - Appendix B: Device Registers and Parameter Block Layout

The document device-driver.md describes the overall device feature proposal and most parts of the detailed design.  

Appendix A lists the standard virtual registers that all devices can implement.

This Appendix B lists the registers that are actually implemented by several existing device drivers.  In some cases the detailed format of the parameter block bodies they expect is given.  The `CDR` namespace is defined for the Common Device Registers in the DeviceDriver.h include file.  The `REG` namespace is defined for the device-specific registers in the `DD<device>.h` include file and the contents can be different for each DeviceDriver.

Appendix C defines the detailed message formats used to communicate between the client and Firmata and back again.

---
##*Hello (DDHello)*

The Hello device driver is a simple example of a device driver.  It generates phrases like "Hello, World", using the strings stored as the interjection (eg, "Hello") and the object (eg "world").  

If the millisecond interval is k\>0 and a read() call has been made with the flag `DAF::MILLI_RUN`, the driver generates a matching `read()` response every k milliseconds.  

###Implemented Registers

Common

    CDR::DriverVersion (r)
    CDR::UnitNamePrefix (r)
    CDR::Stream (r)
    CDR::Intervals (r/w)

Device-specific (Namespace `DDHello::REG`)

    REG::INTERJECTION (r/w)
    REG::OBJECT (r/w)

---
####`CDR::Intervals`
#####Get/set interval timers

*Parameter block buffer*

     0  microsecond interval (LSB)
     1  microsecond interval
     2  microsecond interval
     3  microsecond interval (MSB)
     4  millisecond interval (LSB)
     5  millisecond interval
     6  millisecond interval
     7  millisecond interval (MSB)

---
##*MCP9808 Temperature Sensor (DDMCP9808)*

Note that the values in the parameter blocks for this device are arranged as Little Endian sequences in order to be consistent with the rest of this Device Driver API.  At the lowest level, most of the registers on this device are 16-bit (2-byte) stored in Big Endian order, but the device driver takes care of swapping from host order to device order and vice versa.

If the millisecond interval is k\>0 and a read() call has been made with the flag `DAF::MILLI_RUN`, the driver generates a matching `read()` response every k milliseconds.  

###Implemented Registers

Common

    CDR::DriverVersion (r)
    CDR::UnitNamePrefix (r)
    CDR::Stream (r)
    CDR::Intervals (r/w)

Device-specific (Namespace `DDMCP9808::REG`)

    REG::AMBIENT_TEMP (r)
    REG::MANUF_ID (r)
    REG::DEVICE_ID (r)
    REG::CONFIG (r/w)
    REG::UPPER_TEMP (r/w)
    REG::LOWER_TEMP (r/w)
    REG::CRIT_TEMP (r/w)
    REG::RESOLUTION (r/w)

### Parameter Block Details

---
####`REG::UPPER_TEMP`
#####Get/set upper temperature limit register.
####`REG::LOWER_TEMP`
#####Get/set lower temperature limit register.
####`REG::CRIT_TEMP`
#####Get/set critical temperature limit register.

*Parameter block buffer*

     0  temperature °C (LSB)
     1  temperature °C (MSB)

---
####`CDR::Stream`
#####Get ambient temperature in degrees C.

####`REG::AMBIENT_TEMP`
#####Get ambient temperature in degrees C.

*Parameter block buffer*

     0  temperature °C (LSB)
     1  temperature °C (MSB)

---
####`REG::MANUF_ID`
#####Get Manufacturer ID

*Parameter block buffer*

     0  0x00 (manufacturer ID - LSB)
     1  0x54 (manufacturer ID - MSB)

---
####`REG::DEVICE_ID`
#####Get Device ID

*Parameter block buffer* 

     0  0x04 (device ID - LSB)
     1  0x00 (device ID - MSB)

---
####`REG::RESOLUTION`

#####Get/set sensor resolution

*Parameter block buffer*

     0  0,1,2, or 3 (resolution control bits)

---
##*Meta (DDMeta)*

This DDMeta class is an administrative and development tool.  As a device
driver, it provides an API for access to the DeviceTable (how many devices, device driver versions, etc) and code analysis capabilities (memory usage, timing, etc).

If the millisecond interval is k\>0 and a read() call has been made with the flag `DAF::MILLI_RUN`, the driver generates a matching `read()` response every k milliseconds.  

###Implemented Registers

Common

    CDR::DriverVersion (r)
    CDR::UnitNamePrefix (r)
    CDR::Intervals (r/w)

Device-specific

    REG::AVG_INTERVALS (r)
    REG::DRIVER_COUNT (r)
    REG::DRIVER_VERSION_LIST (r)
    REG::UNIT_NAME_PREFIX_LIST (r)

---

##*Servomotor (DDServo)*

Control the movements of one or more standard servomotors.

If the millisecond interval is k\>0 and a write() call has been made with the flag `DAF::MILLI_RUN`, the driver performs a position command every k milliseconds that moves the connected servo through a simple profile. A response message is sent for each generated command.

Note that right now, this is more in the nature of a demonstration of the continuous write() capability rather than a full feature of the driver.  

###Implemented Registers

####Common

    CDR::DriverVersion (r)
    CDR::UnitNamePrefix (r)
    CDR::Intervals (r/w)

####Device-specific

#####`REG::PIN`
#####Get or set the pin number controlling this servomotor.

*Parameter block buffer*

     0  pin number (LSB)
     1  pin number (MSB)


---
#####`REG::RANGE_MICROSECONDS`
#####Get or set the range in microseconds to use in scaling 0..180 degrees to pulse widths.

The underlying library default min is 544, max is 2400.

*Parameter block buffer*

     0  Min pulse length (microseconds, LSB) (=> 0 degrees)
     1  Min pulse length (microseconds, MSB)
     0  Max pulse length (microseconds, LSB) (=> 180 degrees)
     1  Max pulse length (microseconds, MSB)

---
#####`REG::POSITION_DEGREES`
#####Get or set the current position of the servomotor using angle in degrees.

*Parameter block buffer*

     0  angle in degrees (LSB)
     1  angle in degrees (MSB)

#####`REG::POSITION_MICROSECONDS`
#####Get or set the current position of the servomotor using pulse width in microseconds.

*Parameter block buffer*

     0  angle in microseconds (pulse width) (LSB)
     1  angle in microseconds (pulse width) (MSB)
